@@ -27,6 +27,10 @@
 #include "gconf-property-editor.h"
 #include "theme-thumbnail.h"
 
+typedef void (* ThumbnailGenFunc) (void               *type,
+				   ThemeThumbnailFunc  theme,
+				   AppearanceData     *data,
+				   GDestroyNotify     *destroy);
 
 static gchar *
 find_string_in_model (GtkTreeModel *model, const gchar *value, gint column)
@@ -125,50 +129,43 @@ conv_from_widget_cb (GConfPropertyEditor *peditor, const GConfValue *value)
   return new_value;
 }
 
+static gint
+cursor_theme_sort_func (GtkTreeModel *model,
+                        GtkTreeIter *a,
+                        GtkTreeIter *b,
+                        gpointer user_data)
+{
+  gchar *a_label = NULL;
+  gchar *b_label = NULL;
+  const gchar *default_label;
+  gint result;
+
+  gtk_tree_model_get (model, a, COL_LABEL, &a_label, -1);
+  gtk_tree_model_get (model, b, COL_LABEL, &b_label, -1);
+
+  default_label = _("Default Pointer");
+
+  if (!strcmp (a_label, default_label))
+    result = -1;
+  else if (!strcmp (b_label, default_label))
+    result = 1;
+  else
+    result = strcmp (a_label, b_label);
+
+  g_free (a_label);
+  g_free (b_label);
+
+  return result;
+}
+
 static void
 update_color_buttons_from_string (const gchar *color_scheme, AppearanceData *data)
 {
-  GdkColor color_scheme_colors[6];
-  gchar **color_scheme_strings, **color_scheme_pair, *current_string;
-  gint i;
+  GdkColor color_scheme_colors[8];
   GtkWidget *widget;
 
-  if (!color_scheme || !strcmp (color_scheme, "")) return;
-
-  /* The color scheme string consists of name:color pairs, seperated by
-   * newlines, so first we split the string up by new line */
-
-  color_scheme_strings = g_strsplit (color_scheme, "\n", 0);
-
-  /* loop through the name:color pairs, and save the colour if we recognise the name */
-  i = 0;
-  while ((current_string = color_scheme_strings[i++]))
-  {
-    color_scheme_pair = g_strsplit (current_string, ":", 0);
-
-    if (color_scheme_pair[0] != NULL && color_scheme_pair[1] != NULL)
-    {
-      g_strstrip (color_scheme_pair[0]);
-      g_strstrip (color_scheme_pair[1]);
-
-      if (!strcmp ("fg_color", color_scheme_pair[0]))
-        gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[0]);
-      else if (!strcmp ("bg_color", color_scheme_pair[0]))
-        gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[1]);
-      else if (!strcmp ("text_color", color_scheme_pair[0]))
-        gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[2]);
-      else if (!strcmp ("base_color", color_scheme_pair[0]))
-        gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[3]);
-      else if (!strcmp ("selected_fg_color", color_scheme_pair[0]))
-        gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[4]);
-      else if (!strcmp ("selected_bg_color", color_scheme_pair[0]))
-        gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[5]);
-    }
-
-    g_strfreev (color_scheme_pair);
-  }
-
-  g_strfreev (color_scheme_strings);
+  if (!gnome_theme_color_scheme_parse (color_scheme, color_scheme_colors))
+    return;
 
   /* now set all the buttons to the correct settings */
   widget = glade_xml_get_widget (data->xml, "fg_colorbutton");
@@ -183,23 +180,27 @@ update_color_buttons_from_string (const gchar *color_scheme, AppearanceData *dat
   gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[4]);
   widget = glade_xml_get_widget (data->xml, "selected_bg_colorbutton");
   gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[5]);
+  widget = glade_xml_get_widget (data->xml, "tooltip_fg_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[6]);
+  widget = glade_xml_get_widget (data->xml, "tooltip_bg_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[7]);
 }
 
 static void
 update_color_buttons_from_settings (GtkSettings *settings,
                                     AppearanceData *data)
 {
-  gchar *scheme;
+  gchar *scheme, *setting;
+
   scheme = gconf_client_get_string (data->client, COLOR_SCHEME_KEY, NULL);
+  g_object_get (G_OBJECT (settings), "gtk-color-scheme", &setting, NULL);
+
   if (scheme == NULL || strcmp (scheme, "") == 0)
-  {
     gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), FALSE);
-    g_free (scheme);
-    g_object_get (G_OBJECT (settings), "gtk-color-scheme", &scheme, NULL);
-  }
 
-  update_color_buttons_from_string (scheme, data);
   g_free (scheme);
+  update_color_buttons_from_string (setting, data);
+  g_free (setting);
 }
 
 static void
@@ -250,31 +251,31 @@ static void
 color_button_clicked_cb (GtkWidget *colorbutton, AppearanceData *data)
 {
   gchar *new_scheme;
-  GdkColor colors[6];
-  gchar *bg, *fg, *text, *base, *selected_fg, *selected_bg;
+  GdkColor colors[NUM_SYMBOLIC_COLORS];
+  gchar *str[NUM_SYMBOLIC_COLORS + 1];
+  const gchar *widgets[NUM_SYMBOLIC_COLORS] = {
+      "fg_colorbutton", "bg_colorbutton",
+      "text_colorbutton", "base_colorbutton",
+      "selected_fg_colorbutton", "selected_bg_colorbutton",
+      "tooltip_fg_colorbutton", "tooltip_bg_colorbutton" };
+  const gchar *labels[NUM_SYMBOLIC_COLORS] = {
+      "fg_color", "bg_color",
+      "text_color", "base_color",
+      "selected_fg_color", "selected_bg_color",
+      "tooltip_fg_color", "tooltip_bg_color" };
+  gint i;
   GtkWidget *widget;
 
-  widget = glade_xml_get_widget (data->xml, "fg_colorbutton");
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[0]);
-  widget = glade_xml_get_widget (data->xml, "bg_colorbutton");
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[1]);
-  widget = glade_xml_get_widget (data->xml, "text_colorbutton");
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[2]);
-  widget = glade_xml_get_widget (data->xml, "base_colorbutton");
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[3]);
-  widget = glade_xml_get_widget (data->xml, "selected_fg_colorbutton");
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[4]);
-  widget = glade_xml_get_widget (data->xml, "selected_bg_colorbutton");
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[5]);
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i) {
+    widget = glade_xml_get_widget (data->xml, widgets[i]);
+    gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[i]);
 
-  fg = g_strdup_printf ("fg_color:#%04x%04x%04x\n", colors[0].red, colors[0].green, colors[0].blue);
-  bg = g_strdup_printf ("bg_color:#%04x%04x%04x\n", colors[1].red, colors[1].green, colors[1].blue);
-  text = g_strdup_printf ("text_color:#%04x%04x%04x\n", colors[2].red, colors[2].green, colors[2].blue);
-  base = g_strdup_printf ("base_color:#%04x%04x%04x\n", colors[3].red, colors[3].green, colors[3].blue);
-  selected_fg = g_strdup_printf ("selected_fg_color:#%04x%04x%04x\n", colors[4].red, colors[4].green, colors[4].blue);
-  selected_bg = g_strdup_printf ("selected_bg_color:#%04x%04x%04x", colors[5].red, colors[5].green, colors[5].blue);
+    str[i] = g_strdup_printf ("%s:#%04x%04x%04x",
+             labels[i], colors[i].red, colors[i].green, colors[i].blue);
+  }
+  str[NUM_SYMBOLIC_COLORS] = NULL;
 
-  new_scheme = g_strconcat (fg, bg, text, base, selected_fg, selected_bg, NULL);
+  new_scheme = g_strjoinv ("\n", str);
 
   /* Currently we assume this has only been called when one of the colours has
    * actually changed, so we don't check the original key first */
@@ -282,19 +283,15 @@ color_button_clicked_cb (GtkWidget *colorbutton, AppearanceData *data)
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), TRUE);
 
-  g_free (fg);
-  g_free (bg);
-  g_free (text);
-  g_free (base);
-  g_free (selected_fg);
-  g_free (selected_bg);
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i)
+    g_free (str[i]);
   g_free (new_scheme);
 }
 
 static void
 color_scheme_defaults_button_clicked_cb (GtkWidget *button, AppearanceData *data)
 {
-  gconf_client_set_string (data->client, COLOR_SCHEME_KEY, "", NULL);
+  gconf_client_unset (data->client, COLOR_SCHEME_KEY, NULL);
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), FALSE);
 }
 
@@ -370,6 +367,7 @@ icon_theme_changed (GConfPropertyEditor *peditor,
 			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_ICON));
 }
 
+#ifdef HAVE_XCURSOR
 static void
 update_cursor_size_scale (GnomeThemeCursorInfo *theme,
                           AppearanceData *data)
@@ -434,6 +432,7 @@ update_cursor_size_scale (GnomeThemeCursorInfo *theme,
       gtk_range_set_value (GTK_RANGE (cursor_size_scale), (gdouble) g_array_index (theme->sizes, gint, theme->sizes->len - 1));
   }
 }
+#endif
 
 static void
 cursor_theme_changed (GConfPropertyEditor *peditor,
@@ -447,7 +446,9 @@ cursor_theme_changed (GConfPropertyEditor *peditor,
   if (value && (name = gconf_value_get_string (value)))
     theme = gnome_theme_cursor_info_find (name);
 
+#ifdef HAVE_XCURSOR
   update_cursor_size_scale (theme, data);
+#endif
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "cursor_themes_delete"),
 			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_CURSOR));
@@ -509,6 +510,7 @@ icon_theme_delete_cb (GtkWidget *button, AppearanceData *data)
   generic_theme_delete ("icon_themes_list", THEME_TYPE_ICON, data);
 }
 
+#ifdef HAVE_XCURSOR
 static void
 cursor_size_scale_value_changed_cb (GtkRange *range, AppearanceData *data)
 {
@@ -529,6 +531,7 @@ cursor_size_scale_value_changed_cb (GtkRange *range, AppearanceData *data)
     gconf_client_set_int (data->client, CURSOR_SIZE_KEY, size, NULL);
   }
 }
+#endif
 
 static void
 add_to_treeview (const gchar *tv_name,
@@ -716,6 +719,8 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
   const gchar *key;
   GObject *peditor;
   GConfValue *value;
+  ThumbnailGenFunc generator;
+  ThemeThumbnailFunc thumb_cb;
 
   switch (type)
   {
@@ -723,24 +728,36 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
       themes = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2);
       thumbnail = data->gtk_theme_icon;
       key = GTK_THEME_KEY;
+      generator = (ThumbnailGenFunc) generate_gtk_theme_thumbnail_async;
+      thumb_cb = (ThemeThumbnailFunc) gtk_theme_thumbnail_cb;
       break;
 
     case THEME_TYPE_WINDOW:
       themes = gnome_theme_info_find_by_type (GNOME_THEME_METACITY);
       thumbnail = data->window_theme_icon;
       key = METACITY_THEME_KEY;
+      generator = (ThumbnailGenFunc) generate_metacity_theme_thumbnail_async;
+      thumb_cb = (ThemeThumbnailFunc) metacity_theme_thumbnail_cb;
       break;
 
     case THEME_TYPE_ICON:
       themes = gnome_theme_icon_info_find_all ();
       thumbnail = data->icon_theme_icon;
       key = ICON_THEME_KEY;
+      generator = (ThumbnailGenFunc) generate_icon_theme_thumbnail_async;
+      thumb_cb = (ThemeThumbnailFunc) icon_theme_thumbnail_cb;
       break;
 
     case THEME_TYPE_CURSOR:
       themes = gnome_theme_cursor_info_find_all ();
       thumbnail = NULL;
+#ifdef HAVE_XCURSOR
       key = CURSOR_THEME_KEY;
+#else
+      key = CURSOR_FONT_KEY;
+#endif
+      generator = NULL;
+      thumb_cb = NULL;
       break;
 
     default:
@@ -761,43 +778,18 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
     } else if (type == THEME_TYPE_ICON) {
       name = ((GnomeThemeIconInfo *) l->data)->name;
       label = ((GnomeThemeIconInfo *) l->data)->readable_name;
-    }
-    else if (type == THEME_TYPE_CURSOR) {
+    } else if (type == THEME_TYPE_CURSOR) {
       name = ((GnomeThemeCursorInfo *) l->data)->name;
+      label = ((GnomeThemeCursorInfo *) l->data)->readable_name;
     }
 
     if (!name)
       continue; /* just in case... */
 
-    switch (type)
-    {
-      case THEME_TYPE_GTK:
-        generate_gtk_theme_thumbnail_async ((GnomeThemeInfo *) l->data,
-                                            (ThemeThumbnailFunc) gtk_theme_thumbnail_cb,
-                                            data,
-                                            NULL);
-        break;
-
-      case THEME_TYPE_ICON:
-        generate_icon_theme_thumbnail_async ((GnomeThemeIconInfo *) l->data,
-                                             (ThemeThumbnailFunc) icon_theme_thumbnail_cb,
-                                            data,
-                                            NULL);
-        break;
-
-      case THEME_TYPE_WINDOW:
-        generate_metacity_theme_thumbnail_async ((GnomeThemeInfo *) l->data,
-                                            (ThemeThumbnailFunc) metacity_theme_thumbnail_cb,
-                                            data,
-                                            NULL);
-        break;
-
-      case THEME_TYPE_CURSOR:
-        thumbnail = ((GnomeThemeCursorInfo *) l->data)->thumbnail;
-        break;
-
-      default:
-        break;
+    if (type == THEME_TYPE_CURSOR) {
+      thumbnail = ((GnomeThemeCursorInfo *) l->data)->thumbnail;
+    } else {
+      generator (l->data, thumb_cb, data, NULL);
     }
 
     gtk_list_store_insert_with_values (store, &i, 0,
@@ -806,15 +798,33 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
                                        COL_THUMBNAIL, thumbnail,
                                        -1);
 
-    if (type == THEME_TYPE_CURSOR)
+    if (type == THEME_TYPE_CURSOR && thumbnail) {
       g_object_unref (thumbnail);
+      thumbnail = NULL;
+    }
   }
   g_list_free (themes);
 
+#ifdef HAVE_XCURSOR
+  if (type == THEME_TYPE_CURSOR) {
+    GtkTreeIter i;
+    gtk_list_store_insert_with_values (store, &i, 0,
+                                       COL_LABEL, _("Default Pointer"),
+                                       COL_NAME, "default",
+                                       COL_THUMBNAIL, NULL,
+                                       -1);
+  }
+#endif
+
   sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort_model),
                                         COL_LABEL, GTK_SORT_ASCENDING);
 
+  if (type == THEME_TYPE_CURSOR)
+    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model), COL_LABEL,
+                                     (GtkTreeIterCompareFunc) cursor_theme_sort_func,
+                                     NULL, NULL);
+
   gtk_tree_view_set_model (GTK_TREE_VIEW (list), GTK_TREE_MODEL (sort_model));
 
   renderer = gtk_cell_renderer_pixbuf_new ();
@@ -852,6 +862,7 @@ style_init (AppearanceData *data)
   GtkSettings *settings;
   GtkWidget *w;
   GtkAdjustment *adjustment;
+  gchar *label;
 
   data->gtk_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/gtk-theme-thumbnailing.png", NULL);
   data->window_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/window-theme-thumbnailing.png", NULL);
@@ -864,7 +875,7 @@ style_init (AppearanceData *data)
   prepare_list (data, glade_xml_get_widget (data->xml, "window_themes_list"), THEME_TYPE_WINDOW, (GCallback) window_theme_changed);
   prepare_list (data, glade_xml_get_widget (data->xml, "gtk_themes_list"), THEME_TYPE_GTK, (GCallback) gtk_theme_changed);
   prepare_list (data, glade_xml_get_widget (data->xml, "icon_themes_list"), THEME_TYPE_ICON, (GCallback) icon_theme_changed);
- prepare_list (data, glade_xml_get_widget (data->xml, "cursor_themes_list"), THEME_TYPE_CURSOR, (GCallback) cursor_theme_changed);
+  prepare_list (data, glade_xml_get_widget (data->xml, "cursor_themes_list"), THEME_TYPE_CURSOR, (GCallback) cursor_theme_changed);
 
   w = glade_xml_get_widget (data->xml, "color_scheme_message_hbox");
   gtk_widget_set_no_show_all (w, TRUE);
@@ -875,10 +886,9 @@ style_init (AppearanceData *data)
                                                   GTK_ICON_SIZE_BUTTON));
 
   settings = gtk_settings_get_default ();
-  check_color_schemes_enabled (settings, data);
   g_signal_connect (settings, "notify::gtk-color-scheme", (GCallback) color_scheme_changed, data);
-  update_color_buttons_from_settings (settings, data);
 
+#ifdef HAVE_XCURSOR
   w = glade_xml_get_widget (data->xml, "cursor_size_scale");
   GTK_RANGE (w)->round_digits = 0;
   g_signal_connect (G_OBJECT (w), "value-changed", (GCallback) cursor_size_scale_value_changed_cb, data);
@@ -886,9 +896,21 @@ style_init (AppearanceData *data)
   g_object_set (G_OBJECT (adjustment), "page-size", 0.0, NULL);
 
   w = glade_xml_get_widget (data->xml, "cursor_size_small_label");
-  gtk_label_set_markup (GTK_LABEL (w), g_strdup_printf ("<small><i>%s</i></small>", gtk_label_get_text (GTK_LABEL (w))));
+  label = g_strdup_printf ("<small><i>%s</i></small>", gtk_label_get_text (GTK_LABEL (w)));
+  gtk_label_set_markup (GTK_LABEL (w), label);
+  g_free (label);
+
   w = glade_xml_get_widget (data->xml, "cursor_size_large_label");
-  gtk_label_set_markup (GTK_LABEL (w), g_strdup_printf ("<small><i>%s</i></small>", gtk_label_get_text (GTK_LABEL (w))));
+  label = g_strdup_printf ("<small><i>%s</i></small>", gtk_label_get_text (GTK_LABEL (w)));
+  gtk_label_set_markup (GTK_LABEL (w), label);
+  g_free (label);
+#else
+  w = glade_xml_get_widget (data->xml, "cursor_size_hbox");
+  gtk_widget_set_no_show_all (w, TRUE);
+  gtk_widget_hide (w);
+  gtk_widget_show (glade_xml_get_widget (data->xml, "cursor_message_hbox"));
+  gtk_box_set_spacing (GTK_BOX (glade_xml_get_widget (data->xml, "cursor_vbox")), 12);
+#endif
 
   /* connect signals */
   /* color buttons */
@@ -898,6 +920,8 @@ style_init (AppearanceData *data)
   g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "base_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
   g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "selected_fg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
   g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "selected_bg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
+  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "tooltip_fg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
+  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "tooltip_bg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
   /* revert button */
   g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "color_scheme_defaults_button")), "clicked", (GCallback) color_scheme_defaults_button_clicked_cb, data);
   /* delete buttons */
